import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../provider/data.dart';

class Cell {
  int value;
  final int col;
  final int row;
  final bool isFixed;

  Cell(
    @required this.value,
    @required this.col,
    @required this.row,
    @required this.isFixed,
  );

  Container widget(Data myProvider) {
    String imageAsset = 'assets/skins/empty.png';
    if (this.value > 0) {
      imageAsset = 'assets/skins/' + myProvider.skin + '_' + this.value.toString() + '.png';
    }

    int size = myProvider.size;

    double borderWidth = 3;
    Color borderDarkColor = Colors.black;
    Color borderLightColor = Colors.grey;
    Color borderSelectedColor = Colors.red;
    Color backgroundColor = this.isFixed ? Colors.grey[300] : Colors.grey[100];

    Border borders = Border(
      top: BorderSide(width: borderWidth, color: ((this.row % size) == 0) ? borderDarkColor : borderLightColor),
      left: BorderSide(width: borderWidth, color: ((this.col % size) == 0) ? borderDarkColor : borderLightColor),
      right: BorderSide(width: borderWidth, color: (((this.col + 1) % size) == 0) ? borderDarkColor : borderLightColor),
      bottom: BorderSide(width: borderWidth, color: (((this.row + 1) % size) == 0) ? borderDarkColor : borderLightColor),
    );

    if (this.col == myProvider.currentCellCol && this.row == myProvider.currentCellRow) {
      borders = Border.all(
        color: borderSelectedColor,
        width: borderWidth,
      );
    }

    return Container(
      decoration: BoxDecoration(
        color: backgroundColor,
        border: borders,
      ),
      child: GestureDetector(
        child: Image(
          image: AssetImage(imageAsset),
          fit: BoxFit.fill
        ),
        onTap: () {
          if (this.col != null && this.row != null) {
            if (!this.isFixed && (this.col != myProvider.currentCellCol || this.row != myProvider.currentCellRow)) {
              myProvider.selectCell(this.col, this.row);
            } else {
              myProvider.selectCell(null, null);
            }
          }
        },
      )
    );
  }

  Container widgetUpdateValue(Data myProvider) {
    String imageAsset = 'assets/skins/empty.png';
    if (this.value > 0) {
      imageAsset = 'assets/skins/' + myProvider.skin + '_' + this.value.toString() + '.png';
    }

    return Container(
      decoration: BoxDecoration(
        color: Colors.grey[200],
        border: Border.all(
          color: Colors.black,
          width: 2,
        ),
      ),
      child: GestureDetector(
        child: Image(
          image: AssetImage(imageAsset),
          fit: BoxFit.fill
        ),
        onTap: () {
          if (this.col != null && this.row != null) {
            myProvider.updateCellValue(this.col, this.row, this.value);
          }
          myProvider.selectCell(null, null);
        },
      )
    );
  }
}
